@@ -11,6 +11,12 @@ import maxdot
 import csv
 from StringIO import StringIO
 
+H5PY_SUPPORT = True
+try:
+    import h5py
+except ImportError:
+    H5PY_SUPPORT = False
+
 def median(values):
     values = sorted(values)
     if len(values) % 2 == 1:
@@ -48,6 +54,10 @@ class NamedMatrix(object):
         else:
             self.colnames = list(colnames)
         
+        # We expect the row and column names to be strings.
+        self.rownames = map(str, self.rownames)
+        self.colnames = map(str, self.colnames)
+        
         self._row_index = None
         self._column_index = None
         # Needed to make NamedMatrix iterable.
@@ -178,6 +188,15 @@ class NamedMatrix(object):
             raise Exception("Dimension mismatch, self.columns must equal other.rownames.")
         return NamedMatrix(self._data*other._data, self.rownames, other.colnames)
     
+    def __eq__(self, other):
+        if not self.rownames == other.rownames:
+            return False
+        if not self.colnames == other.colnames:
+            return False
+        if not list(self) == list(other):
+            return False
+        return True
+    
     def argsort(self):
         """ Return the indices that would sort the array """
         return numpy.argsort(self._data)
@@ -297,6 +316,9 @@ class NamedMatrix(object):
     @staticmethod
     def save_to_hdf5(file, named_matrix):
         """Save named_matrix to an hdf5 file"""
+        if not H5PY_SUPPORT:
+            raise Exception("You must install the h5py package to use hdf5 file format.")
+        
         f5 = h5py.File(file,'w')
         f5['rownames'] = numpy.array(named_matrix.rownames)
         f5['colnames'] = numpy.array(named_matrix.colnames)
@@ -306,18 +328,51 @@ class NamedMatrix(object):
     @staticmethod
     def load_from_hdf5(file):
         """Load named_matrix from an hdf5 file"""
-        import h5py
+        if not H5PY_SUPPORT:
+            raise Exception("You must install the h5py package to use hdf5 file format.")
+        
         f5 = h5py.File(file,'r')
-
         d = f5['_data']
         r = f5['rownames']
         c = f5['colnames']
         result = NamedMatrix(d,rownames=r,colnames=c)
         f5.close()
         return result
-
+    
     @staticmethod
-    def load_from_file(file, sourcecode = None, has_header=True, delimiter=','):
+    def load_from_matrix_file(file, num_rows, has_header=True, delimiter=','):
+        """
+        Load a matrix from a file which contains num_rows rows and num_col columns.
+        """
+        if type(file) is str:
+            file = open(file)
+        
+        reader = csv.reader(file, delimiter=delimiter)
+        header = reader.next()
+        colnames = header[1:]
+        rownames = list()
+        
+        mat = NamedMatrix(None, range(num_rows), colnames)
+        
+        for row in reader:
+            if not len(row[1:]) == len(colnames):
+                raise Exception("Wrong number of columns in row: %d, had %d, expected %d.",
+                    (len(rownames), len(row[1:]), len(colnames)))
+            mat[len(rownames),:] = map(float, row[1:])
+            rownames.append(row[0])
+        
+        mat.rownames = rownames
+        
+        if not sorted(rownames) == rownames:
+            mat = mat.filter(rows=sorted(rownames))
+        
+        if not sorted(colnames) == colnames:
+            mat = mat.filter(columns=sorted(colnames))
+        
+        return mat
+    
+    @staticmethod
+    def load_from_file(file, sourcecode = None, has_header=True, delimiter=',', matrix=False):
         """Load a matrix from a file.  Any missing values are assumed to 
            be 0.  The row and column names are sorted and the shape of
            the matrix is determined by the number of rows (1st column)
